module SecIO where

import Lattice
import Sec

-- SecIO
newtype SecIO s a = MkSecIO (IO (Sec s a))


instance Functor (SecIO s) where
  h `fmap` (MkSecIO io) = MkSecIO ( do sec <- io
                                       return (h `fmap` sec) )


instance Monad (SecIO s) where
  return x =
    MkSecIO (return (return x))
  
  MkSecIO m >>= k =
    MkSecIO (do sa <- m
                let MkSecIO m' = k (reveal sa)
                m')

-- SecIO functions
value :: Sec s a -> SecIO s a
value sa = MkSecIO (return sa)

run :: SecIO s a -> IO (Sec s a)
run (MkSecIO m) = m

plug :: Less sl sh => SecIO sh a -> SecIO sl (Sec sh a)
plug secio_sh@(MkSecIO m) = less sl sh `seq` secio_sl
                            where 
                                 (secio_sl) = MkSecIO (do sha <- m
                                                          return (sec sha))
                                 sl = unSecIOType secio_sl
                                 sh = unSecIOType secio_sh

-- Internal function, not exported. For type-checking purposes.
unSecIOType :: SecIO s a -> s   
unSecIOType _ = undefined


-- File IO
data File s
  = MkFile FilePath
 
readSecIO :: File s' -> SecIO s (Sec s' String)
readSecIO (MkFile file) =
  MkSecIO ((sec . sec) `fmap` readFile file)

writeSecIO :: File s' -> String -> SecIO s ()
writeSecIO (MkFile file) s =
  MkSecIO (sec `fmap` writeFile file s)


-- Derived operations
s_read :: Less s' s => File s' -> SecIO s String
s_read file =
  do ss <- readSecIO file
     value (up ss)

s_write :: Less s s' => File s' -> String -> SecIO s (Sec s' ())
s_write file s = undefined --plug (writeSecIO file s)
